﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Digiwin.Common.Torridity.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DecimalDemo {
    class Program {
        static void Main(string[] args) {
            TestDecimal();
        }

        public static void TestDecimal() {
            var w = new DecimalEncoder();

            decimal[] testValues = { };
            WriteAndRead<decimal>(w, testValues, 1);

            testValues = new decimal[] { 0m, 0m, 0m, 0m };
            WriteAndRead<decimal>(w, testValues, 1);

            testValues = new decimal[] { 0m, 1.0m, 2.0m, 3.0m };
            WriteAndRead<decimal>(w, testValues, 1 + (1 + 1 + 1 + 1));

            testValues = new decimal[] { 0m, 1.1m, 2.0m, 3.0m };//出现了一个小数
            WriteAndRead<decimal>(w, testValues, 1 + (1 + 3 + 2 + 2));

            testValues = new decimal[] { 0m, -1.0m, 2.0m, 3.0m };//出现了一个负数
            WriteAndRead<decimal>(w, testValues, 1 + (1 + 2 + 2 + 2));

            //基本值
            testValues = new decimal[] { 
                0m,             //1
                0.1m,           //1+1+1
                1.0m,           //1+1
                1.1m,           //1+1+1
                1.01m,          //1+1+1
                1.001m,         //1+1+1
                12.0012m,       //1+1+1
                -0.1m,          //1+1+1
                -1m,            //1+1
                -1.1m,          //1+1+1
                -1.01m,         //1+1+1
                -1.001m,        //1+1+1
                -12.0012m       //1+1+1
            };
            WriteAndRead<decimal>(w, testValues, 1 + (1 + 3 + 2 + 3 + 3 + 3 + 3 + 3 + 2 + 3 + 3 + 3 + 3));

            //过载
            testValues = new decimal[] { 
                Int32.MaxValue,             //1+5
                Int32.MinValue,             //1+4*4
                decimal.MaxValue,           //1+4*4
                decimal.MinValue,           //1+4*4
                3.0123456789m,              //1+4*4
                Int32.MaxValue - 1,         //1+5
                Int32.MinValue + 1,         //1+5
                Int32.MaxValue - 0.0123456789m, //1+4*4
                Int32.MinValue + 0.0123456789m, //1+4*4
                4.999999999m                    //1+1+5
            };
            WriteAndRead<decimal>(w, testValues, 1 + (6 + 17 + 17 + 17 + 17 + 6 + 6 + 17 + 17 + 7));

            var sw = System.Diagnostics.Stopwatch.StartNew();

            testValues = new decimal[10000];
            using (var stream = new System.IO.MemoryStream()) {
                SerializeContext context = new SerializeContext(stream);
                for (int i = 0; i < 10000; i++) {
                    for (int j = 0; j < 10000; j++) {
                        testValues[j] = Convert.ToDecimal(j) + 0.1m;
                    }
                    w.Write(context, testValues);
                    stream.Position = 0;
                    w.Read(context, testValues.Length);
                    stream.Position = 0;
                }
            }
            sw.Stop();
            Console.WriteLine(sw.Elapsed);
        }

        static void WriteAndRead<T>(DataEncoder w, T[] values, long bytesCount) {

            T[] newValues;

            using (var stream = new System.IO.MemoryStream()) {
                SerializeContext context = new SerializeContext(stream);
                w.Write(context, values);
                Assert.AreEqual<long>(bytesCount, stream.Length); //期望的字节数
                stream.Position = 0;

                newValues = (T[])w.Read(context, values.Length);
            }

            Assert.AreEqual<int>(values.Length, newValues.Length);
            for (int i = 0; i < values.Length; i++) {
                Assert.AreEqual<T>(values[i], newValues[i]);
            }
        }

    }
}
